"""
This file includes the functions for reading data, etc.
"""
import os
import sys
import six.moves.cPickle as pickle
import numpy as np
import cv2 as cv
import matplotlib.pyplot as plt

def preprocess_data_target():
    data_dir_path = "./TestData/Original/"
    target_dir_path = "./GroundTruth/Original/"
    data = []
    target = []
    n_types_target = -1
    dump_name = 'dataset'
    img_size_h = 240  
    img_size_w = 320

    data_list = os.listdir(data_dir_path)
    target_list = os.listdir(target_dir_path)

    for i in range(0,len(data_list)):
        root_data, ext_data = os.path.splitext(data_list[i])
        root_target, ext_target = os.path.splitext(target_list[i])
        data_name_tmp = root_data[0:len(root_data)-7]+root_data[len(root_data)-4:len(root_data)]
        target_name_tmp = root_target[0:len(root_target)-6]+root_target[len(root_target)-4:len(root_target)]
        if data_name_tmp != target_name_tmp:
            print data_name_tmp,"\t",target_name_tmp
            sys.exit()

    for i in range(0,len(data_list)):
        print i,'\t',data_list[i],'\t',target_list[i]
        data_img,dataF_img = image_read(data_dir_path+data_list[i],img_size_w,img_size_h)
        target_img,targetF_img = image_readgray(target_dir_path+target_list[i],img_size_w,img_size_h)
        pickle.dump((data_img,data_list[i],0), open("./TestData/"+makefilename(2*i,5), 'wb'), -1)
        pickle.dump((dataF_img,data_list[i],1), open("./TestData/"+makefilename(2*i+1,5), 'wb'), -1)
        pickle.dump((target_img,target_list[i],0), open("./GroundTruth/"+makefilename(2*i,5), 'wb'), -1)
        pickle.dump((targetF_img,target_list[i],1), open("./GroundTruth/"+makefilename(2*i+1,5), 'wb'), -1)

def load_data_batch(file_list,data_dirpath,sal_dirpath):
    data_batch,sal_batch = [],[]
    for name in file_list:
        tmp = []
        data_img,img_name,data_type = load_data(data_dirpath+name)
        data_batch.append(data_img)
        data_img,img_name,data_type = load_data(sal_dirpath+name)
        tmp.append(data_img)
        sal_batch.append(tmp)
    return np.array(data_batch,np.float32), np.array(sal_batch,np.float32)
    
def load_test_batch(file_list,data_dirpath,sal_dirpath):
    data_batch,sal_batch,imgname_batch,salname_batch = [],[],[],[]
    for name in file_list:
        tmp = []
        data_img,img_name,data_type = load_data(data_dirpath+name)
        data_batch.append(data_img)
        imgname_batch.append(img_name)
        data_img,img_name,data_type = load_data(sal_dirpath+name)
        tmp.append(data_img)
        sal_batch.append(tmp)
        salname_batch.append(img_name)
    return np.array(data_batch,np.float32), np.array(sal_batch,np.float32), imgname_batch, salname_batch

def load_data(filename):
    data_img, img_name, data_type = pickle.load(open(filename, 'rb'))
    return data_img, img_name, data_type


def image_read(file_name,img_size_w,img_size_h):
    root, ext = os.path.splitext(file_name) 
    if ext == '.png' or ext == '.jpg' or ext == '.jpeg' or ext == '.bmp':
        image = cv.imread(file_name,cv.IMREAD_COLOR)
        image_flip = cv.flip(image,1)
        image = cv.resize(image, (img_size_w, img_size_h))
        image = image.transpose(2,0,1) 
        
        image_flip = cv.resize(image_flip, (img_size_w, img_size_h)) 
        image_flip = image_flip.transpose(2,0,1)
        image = image/255.
        image_flip = image_flip/255.
        return image,image_flip

def image_readgray(file_name,img_size_w,img_size_h):
    root, ext = os.path.splitext(file_name) 
    
    if ext == '.png' or ext == '.jpg' or ext == '.jpeg' or ext == '.bmp':
        image = cv.imread(file_name,cv.IMREAD_GRAYSCALE)
        image_flip = cv.flip(image,1)
        image = cv.resize(image, (img_size_w, img_size_h))
        
        image_flip = cv.resize(image_flip, (img_size_w, img_size_h)) 
        image = image/255.
        image_flip = image_flip/255.
        return image,image_flip
        

def makefilename(num,size):
    d = len(str(num))
    if d == size:
        return str(num)
    elif d > size:
        sys.exit()
    name = "0"
    for i in range(0,size-d-1):
        name += "0"
    return name+str(num)


def save_saliency(data_array,filename):
    for i in range(0,len(data_array)):
        pickle.dump(data_array[i], open(filename+"_"+str(i)+'.data', 'wb'), -1)

def draw_saliency(data):
    fig, ax = plt.subplots()

    image = data.data
    ax.imshow(image, interpolation='hamming')
    ax.set_xticklabels("")
    ax.set_yticklabels("")
    ax.tick_params(bottom=False,left=False)
    
    plt.gray()
    plt.show()
    
